#!/usr/bin/env python

from email import message_from_file
from vCalendar import parse
import sys

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

# Postfix exit codes.

EX_USAGE        = 64
EX_DATAERR      = 65
EX_NOINPUT      = 66
EX_NOUSER       = 67
EX_NOHOST       = 68
EX_UNAVAILABLE  = 69
EX_SOFTWARE     = 70
EX_OSERR        = 71
EX_OSFILE       = 72
EX_CANTCREAT    = 73
EX_IOERR        = 74
EX_TEMPFAIL     = 75
EX_PROTOCOL     = 76
EX_NOPERM       = 77
EX_CONFIG       = 78

# Permitted iTIP content types.

itip_content_types = [
    "text/calendar",                        # from RFC 6047
    "text/x-vcalendar", "application/ics",  # other possibilities
    ]

def process(f):
    msg = message_from_file(f)

    # Handle messages with iTIP parts.

    for part in msg.walk():
        if part.get_content_type() in itip_content_types and \
            part.get_param("method"):

            handle_itip_part(part)

def get_itip_elements(elements):
    d = {}
    for name, attr, value in elements:
        if isinstance(value, list):
            d[name] = attr, get_itip_elements(value)
        else:
            d[name] = attr, value
    return d

def get_value(d, name):
    if d.has_key(name):
        attr, value = d[name]
        return value
    else:
        return None

def handle_itip_part(part):
    method = part.get_param("method")

    f = StringIO(part.get_payload(decode=True))
    doctype, attrs, elements = parse(f, encoding=part.get_content_charset())

    if doctype == "VCALENDAR":
        itip = get_itip_elements(elements)

        if get_value(itip, "METHOD") == method:
            for name, handler in [
                ("VFREEBUSY", handle_itip_freebusy),
                ("VEVENT", handle_itip_event),
                ("VTODO", handle_itip_todo),
                ("VJOURNAL", handle_itip_journal),
                ]:

                obj = get_value(itip, name)
                if obj:
                    uid = get_value(obj, "UID")
                    if uid:
                        handler(obj, method)

def handle_itip_freebusy(freebusy, method):
    print >>open("/tmp/imip.txt", "a"), freebusy

def handle_itip_event(event, method):
    print >>open("/tmp/imip.txt", "a"), event

def handle_itip_todo(todo, method):
    print >>open("/tmp/imip.txt", "a"), todo

def handle_itip_journal(journal, method):
    print >>open("/tmp/imip.txt", "a"), journal

if __name__ == "__main__":
    process(sys.stdin)

# vim: tabstop=4 expandtab shiftwidth=4
